#include "decisiontree.h"
# include <QDebug>
DecisionTree::DecisionTree(DiscreteDataset *tr,DiscreteDataset *tt)
{
    trainSet=tr;
    testSet=tt;
    mainTree=NULL;
}

Tree        *DecisionTree::subTree(QVector<int> &selectedFeatures,
                                   DiscreteDataset *dataset,int feature)
{
    Tree *T=new Tree(dataset->getheader()[feature],NODE_FEATURE);
    selectedFeatures<<feature;
     QStringList values=dataset->enumerateValues(feature);
     for(int i=0;i<values.size();i++)
     {

         Tree *sub=new Tree(values[i],NODE_VALUE);
         T->addChild(sub);
         DiscreteDataset *subDataset=dataset->subDataset(feature,values[i]);
         QStringList mclass=subDataset->getClass();
         if(mclass.size()==1)//is leaf
         {
             Tree *leaf=new Tree(mclass[0],NODE_VALUE);
             sub->addChild(leaf);
         }
         else
         {
            //choose another feature not in list selectedFeatures
            int anotherFeature;
             do
             {
                anotherFeature=rand() % dataset->features();
                if(!selectedFeatures.contains(anotherFeature)) break;
             }while(true);
            QVector<int> copySelected=selectedFeatures;
            Tree *newTree=subTree(copySelected,subDataset,anotherFeature);
            sub->addChild(newTree);
         }
         delete subDataset;
     }
     return T;
}

void    DecisionTree::train()
{
    QVector<int> selectedFeatures;
    int startFeature=rand() % trainSet->features();
    mainTree=subTree(selectedFeatures,trainSet,startFeature);
    mainTree->print();
}

QString DecisionTree::getClass(Pattern x)
{
    return followTree(x,mainTree);
}

double      DecisionTree::getTestError()
{
    if(testSet==NULL) return 100.0;
    int missed=0;
    for(int i=0;i<testSet->getpoints();i++)
    {
        Pattern x=testSet->getx(i);
        QString y=testSet->gety(i);
        QString esty=getClass(x);
        if(y!=esty) missed++;
    }
    return missed*100.0/testSet->getpoints();
}

QString     DecisionTree::followTree(Pattern x,Tree *T)
{
    if(T->isLeaf()) return T->getvalue();
    else
    {
        //is value
        QString v=T->getvalue();
        if(T->getType()==NODE_VALUE)
        {
            return followTree(x,T->getChild(0));
        }
        //is feature
        int pos=trainSet->getfeature(v);
        for(int i=0;i<T->nchildren();i++)
        {
            Tree *subTree=T->getChild(i);
            if(subTree->getvalue()==x[pos])
                return followTree(x,subTree);
        }
    }
    return "";
}

DecisionTree::~DecisionTree()
{
    delete mainTree;
}
